/*
 *  xfce4-smartbookmark-plugin
 *  Copyright (C) 2005 Emanuele Rocca <ema@debian.org>
 *
 *  lot of code borrowed from:
 *  xfce4-minicmd-plugin
 *  Copyright (C) 2003 Biju Philip Chacko (botsie@users.sf.net)
 *  Copyright (C) 2003 Eduard Roccatello (master@spine-group.org)
 *
 *  and from:
 *  xfce4-netload-plugin
 *  Copyright (C) 2005 Benedikt Meurer <benedikt.meurer@unix-ag.uni-siegen.de>
 *  Copyright (C) 2005 Bernhard Walle  <bernhard.walle@gmx.de>
 *  Copyright (C) 2005 Hendrik Scholz  <hscholz@raisdorf.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <X11/Xlib.h>
#include <X11/Xatom.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include <libxfce4util/i18n.h>
#include <libxfce4util/util.h>
#include <libxfce4util/debug.h>

#include <libxfcegui4/dialogs.h>
#include <panel/plugins.h>
#include <panel/xfce.h>

/*
 * Types 
 */
typedef struct {
    GtkWidget *ebox;
    GtkWidget *entry;           /* keyword entry */
	GtkWidget *label;

	/* options */
	gchar *label_text;
	gchar *url;
	gint size;
	
	/* options dialog */
	GtkWidget *opt_dialog;
	/* entries (options dialog) */
	GtkWidget *label_entry;
	GtkWidget *url_entry;
	GtkWidget *size_spinner;
} t_search;

static gboolean do_search(const char *url, const char *keyword)
{
	gchar *execute;
	gboolean success;
	
	execute = g_strconcat("x-www-browser \"", url, NULL);
	execute = g_strconcat(execute, keyword, NULL);
	execute = g_strconcat(execute, "\"", NULL);

	success = exec_command(execute);
	g_free(execute);

	return success;
}

/* redraw the plugin */
static void update_search(t_search *search) {
		gtk_widget_hide(GTK_WIDGET(search->ebox));
		gtk_widget_hide(search->label);
		gtk_label_set_text(GTK_LABEL(search->label), search->label_text); 
		gtk_widget_show(GTK_WIDGET(search->ebox));
		gtk_widget_show(search->label);
}

/* callback: apply the new values to: url, label_text, size */
static void search_apply_options_cb(GtkWidget *widget, t_search *search)
{
	search->url = g_strdup(gtk_entry_get_text(GTK_ENTRY(search->url_entry)));
	
	search->label_text = g_strdup(gtk_entry_get_text(GTK_ENTRY(search->label_entry)));
	
	search->size = (gint)(gtk_spin_button_get_value(GTK_SPIN_BUTTON(search->size_spinner)));
    gtk_entry_set_width_chars(GTK_ENTRY(search->entry), search->size);

	update_search(search);
}

/* callback: apply the new value to the url string */
static void url_entry_activate_cb(GtkWidget *widget, t_search *search)
{
	search->url = g_strdup(gtk_entry_get_text(GTK_ENTRY(search->url_entry)));
	
	update_search(search);
}

/* callback: apply the new value to the label_text string */
static void text_entry_activate_cb(GtkWidget *widget, t_search *search)
{
	search->label_text = g_strdup(gtk_entry_get_text(GTK_ENTRY(search->label_entry)));

	update_search(search);
}

static gboolean entry_buttonpress_cb(GtkWidget *entry, GdkEventButton *event, gpointer data)
{
	static Atom atom = 0;
	GtkWidget *toplevel = gtk_widget_get_toplevel (entry);

	if (event->button != 3 && toplevel && toplevel->window) {
		XClientMessageEvent xev;

		if (G_UNLIKELY(!atom))
			atom = XInternAtom (GDK_DISPLAY(), "_NET_ACTIVE_WINDOW", FALSE);

		xev.type = ClientMessage;
		xev.window = GDK_WINDOW_XID (toplevel->window);
		xev.message_type = atom;
		xev.format = 32;
		xev.data.l[0] = 0;
		xev.data.l[1] = 0;
		xev.data.l[2] = 0;
		xev.data.l[3] = 0;	
		xev.data.l[4] = 0;	

		XSendEvent (GDK_DISPLAY (), GDK_ROOT_WINDOW (), False,
					StructureNotifyMask, (XEvent *) & xev);

                gtk_widget_grab_focus (entry);
	}

        return FALSE;
}

/* callback: called when a button is pressed into the main entry */
static gboolean entry_keypress_cb(GtkWidget *entry, GdkEventKey *event, t_search *search)
{
    const gchar *key = NULL;   /* keyword */

    switch (event->keyval) {
        case GDK_Return:
            key = gtk_entry_get_text(GTK_ENTRY(entry));

            if (do_search(search->url, key)) {
                gtk_entry_set_text(GTK_ENTRY(entry), "");  /* clear the entry */
            }
            return TRUE;
        default:
            /* hand over to default signal handler */
            return FALSE;
    }
}

static t_search *search_new(void)
{
    t_search *search;
	GtkWidget *box, *align;

    search = g_new(t_search, 1);

    search->ebox = gtk_event_box_new();
	align = gtk_alignment_new(0.5, 0.5, 0, 0);
    gtk_container_add(GTK_CONTAINER(search->ebox), align);

	box = gtk_vbox_new(FALSE, 0);
	
	/* default options */
	search->url = "http://bugs.debian.org/";
	search->label_text = "BTS";
	search->size = 5;

    gtk_container_add(GTK_CONTAINER(align), box);

    search->entry = gtk_entry_new();
    gtk_entry_set_width_chars(GTK_ENTRY(search->entry), search->size);
	
	search->label = gtk_label_new(search->label_text);
    gtk_box_pack_start(GTK_BOX(box), search->label, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box), search->entry, FALSE, FALSE, 0);
    // g_signal_connect(command->entry, "activate", G_CALLBACK(runcl), command);
    g_signal_connect(search->entry, "key-press-event", G_CALLBACK(entry_keypress_cb), search);

    g_signal_connect (search->entry, "button-press-event", G_CALLBACK(entry_buttonpress_cb), NULL);
    	
	gtk_widget_show_all(search->ebox);

    return (search);
}

static gboolean search_control_new(Control * ctrl)
{
    t_search *search;

    search = search_new();
	
    gtk_container_add(GTK_CONTAINER(ctrl->base), search->ebox);

    ctrl->data = (gpointer) search;
    ctrl->with_popup = FALSE;

    gtk_widget_set_size_request(ctrl->base, -1, -1);

    return (TRUE);
}

static void search_free(Control * ctrl)
{
    t_search *search;

    g_return_if_fail(ctrl != NULL);
    g_return_if_fail(ctrl->data != NULL);

    search = (t_search *) ctrl->data;

    g_free(search);
}

static void search_read_config(Control * ctrl, xmlNodePtr parent)
{
	xmlChar *value;
	xmlNodePtr child;

	t_search *t = (t_search *) ctrl->data;

	if (!parent || !parent->children) return;

	child = parent->children;

	if (!xmlStrEqual(child->name, "WebSearch")) return;
	if ((value = xmlGetProp(child, (const xmlChar *) "url")) != NULL)
	{
		t->url = g_strdup((gchar *)value);
	}
	if ((value = xmlGetProp(child, (const xmlChar *) "label_text")) != NULL)
	{
		t->label_text = g_strdup((gchar *)value);
	}
	if ((value = xmlGetProp(child, (const xmlChar *) "size")) != NULL)
	{
		t->size = atoi(value);
	}
	
	update_search(t);
}

static void search_write_config(Control * ctrl, xmlNodePtr parent)
{
	xmlNodePtr root;
	char value[20];

	t_search *t = (t_search *) ctrl->data;

	root = xmlNewTextChild(parent, NULL, "WebSearch", NULL);
	
	/* url */
	xmlSetProp(root, "url", t->url);
	
	/* label */
	if(t->label_text)
	{
		xmlSetProp(root, "label_text", t->label_text);
	}

	/* size */
    g_snprintf(value, 20, "%d", t->size);
    xmlSetProp(root, "size", value);
}

static void search_attach_callback(Control * ctrl, const gchar * signal,
                                    GCallback cb, gpointer data)
{
    t_search *search;

    search = (t_search *) ctrl->data;
    g_signal_connect(search->ebox, signal, cb, data);
    g_signal_connect(search->entry, signal, cb, data);
}

static void search_set_size(Control * ctrl, int size)
{
    /* do the resize of entry :) */
};

/* options dialog */
static void search_create_options(Control * ctrl, GtkContainer * container, GtkWidget * done)
{
	t_search *search;
	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);

	GtkWidget *urllabel = gtk_label_new(_("URL:  "));
	gtk_widget_show(urllabel);
	
	GtkWidget *textlabel = gtk_label_new(_("Label:"));
	gtk_widget_show(textlabel);

	GtkWidget *sizelabel = gtk_label_new(_("Size:"));
	gtk_widget_show(sizelabel);
	search = (t_search *) ctrl->data;

	GtkAdjustment *spinner_adj = (GtkAdjustment *) gtk_adjustment_new (search->size, 2.0, 10.0, 1.0, 5.0, 5.0);
	
	search->opt_dialog = gtk_widget_get_toplevel(done);

	/* url entry */
	search->url_entry = gtk_entry_new();
    gtk_entry_set_width_chars(GTK_ENTRY(search->url_entry), 32);
	gtk_widget_show(search->url_entry);
	/* text entry */
	search->label_entry = gtk_entry_new();
	gtk_widget_show(search->label_entry);
	
	gtk_label_set_use_markup(GTK_LABEL(urllabel), TRUE);
	
	/* url label */
	gtk_box_pack_start(GTK_BOX(hbox), urllabel, FALSE, FALSE, 5);
	/* url field */
	if(search->url!=NULL)
	    gtk_entry_set_text(GTK_ENTRY(search->url_entry), search->url);
    gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(search->url_entry), FALSE, FALSE, 0);
    g_signal_connect (GTK_WIDGET(search->url_entry), "activate", G_CALLBACK (url_entry_activate_cb), search);
	
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
	gtk_widget_show(hbox);
	/*gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);*/

	hbox = gtk_hbox_new(FALSE, 0);
	/* text label */
	gtk_box_pack_start(GTK_BOX(hbox), textlabel, FALSE, FALSE, 5);
	/* text field */
	if(search->label_text!=NULL) 
	    gtk_entry_set_text(GTK_ENTRY(search->label_entry), search->label_text);
    gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(search->label_entry), FALSE, FALSE, 0);

	/* size label */
	gtk_box_pack_start(GTK_BOX(hbox), sizelabel, FALSE, FALSE, 5);
	/* size spinner */
    search->size_spinner = gtk_spin_button_new(spinner_adj, 1.0, 0);
	gtk_box_pack_start(GTK_BOX(hbox), search->size_spinner, FALSE, FALSE, 0);
	gtk_widget_show(search->size_spinner);
	
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
	gtk_widget_show(hbox);

    g_signal_connect (GTK_WIDGET(search->label_entry), "activate", G_CALLBACK (text_entry_activate_cb), search);
	/* done button */
    g_signal_connect(GTK_WIDGET(done), "clicked", G_CALLBACK(search_apply_options_cb), search);
	/*
	gtk_box_pack_start(GTK_BOX(vbox), cpulabel, FALSE, FALSE, 0);
	*/

	gtk_container_add(container, vbox);

	gtk_widget_show(vbox);
}

/* initialization */
G_MODULE_EXPORT void xfce_control_class_init(ControlClass * cc)
{
    xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");

    /* these are required */
    cc->name = _("smartbookmark");
    cc->caption = _("Smart Bookmark");

    cc->create_control = (CreateControlFunc) search_control_new;

    cc->free = search_free;
    cc->attach_callback = search_attach_callback;

    /* options */
    cc->read_config = search_read_config;
    cc->write_config = search_write_config;
    cc->create_options = search_create_options; 
	
	/* resize */
    cc->set_size = search_set_size;
}

/* required! defined in panel/plugins.h */
XFCE_PLUGIN_CHECK_INIT
